import {EditorState, Plugin} from 'prosemirror-state'
import {EditorView} from 'prosemirror-view'
import {history} from 'prosemirror-history'
import {keymap} from 'prosemirror-keymap'
import {baseKeymap} from 'prosemirror-commands'
import {DOMParser, DOMSerializer} from 'prosemirror-model'

import {Menu, blocks, marks} from './godo-menus.mjs'
import {dialog} from './godo-dialog.mjs'
import {loadSchema, buildKeymap} from './utils.mjs'
import base from './components/base.mjs'
import fontMarks from './components/font-marks.mjs'
import heading from './components/heading.mjs'
import link from './components/link.mjs'
import list from './components/list.mjs'

function menuPlugin (menu) {
  return new Plugin({
    view (editorView) {
      let menuView = new Menu(menu, editorView)
      editorView.dom.parentNode.insertBefore(menuView.el, editorView.dom)
      return menuView
    },
  })
}

function changeEventPlugin (editor) {
  return new Plugin({
    state: {
      init () {},
      apply (tr, callback) {
        if (tr.docChanged) {
          clearTimeout(callback)
          callback = setTimeout(() => {
            editor.textarea.value = editor.getHTML(tr.doc.content)
            editor.textarea.dispatchEvent(new Event('change', { 'bubbles': true }))
          }, 300)
        }
        return callback
      },
    },
  })
}

export default class Godo {
  constructor (textarea, options) {
    this.textarea = textarea
    this.textarea.hidden = true
    this.textarea.style.display = 'none'

    this.defaultOptions = {
      schema: 'full',
      instantUpdate: false,
      updateEvent: false,
    }

    this.options = Object.assign(this.defaultOptions, options)

    const componentsDefinitions = [base, fontMarks, link]
    if (this.options.schema === 'full') {
      componentsDefinitions.push(heading, list)
    }

    const components = componentsDefinitions.map(c => c(options))
    this.schema = loadSchema(components)

    this.editorWrapper = document.createElement('div')
    this.editorWrapper.className = 'godo'
    textarea.insertAdjacentElement('afterend', this.editorWrapper)

    this.marksMenu = menuPlugin(marks(components, this.schema))
    this.blocksMenu = menuPlugin(blocks(components, this.schema))
    this.menuDialog = document.body.appendChild(dialog)

    let pluginsList = [
      history(),
      keymap(buildKeymap(components, this.schema, null)),
      keymap(baseKeymap),
      this.marksMenu,
    ]

    if (this.options.schema !== 'basic' && !textarea.readOnly) pluginsList.push(this.blocksMenu)
    if (options.instantUpdate) pluginsList.push(changeEventPlugin(this))

    const stateConfig = () => {
      return {
        schema: this.schema,
        doc: DOMParser.fromSchema(this.schema).parse(this.getTextAreaContent()),
        plugins: pluginsList,
      }
    }

    this.state = EditorState.create(stateConfig())

    const label = textarea.id ? document.querySelector(`label[for=${textarea.id}]`) : undefined
    this.view = new EditorView(this.editorWrapper, {
      state: this.state,
      editable: () => !textarea.readOnly,
      attributes: {
        class: 'godo--editor',
        role: 'textbox',
        'aria-multiline': 'true',
        'aria-labelledby': label ? label.id : undefined,
      },
    })

    if (this.textarea.form) {
      this.textarea.form.addEventListener('submit', () => {
        textarea.value = this.getHTML(this.view.state.doc.content)
      })
    }

    const updateEvent = this.options.updateEvent
    if (updateEvent) {
      this.textarea.addEventListener(updateEvent, () => {
        this.state = EditorState.create(stateConfig())
        this.view.updateState(this.state)
      })
    }
  }

  getTextAreaContent () {
    const w = document.createElement('div')
    w.innerHTML = this.textarea.value
    return w
  }

  getHTML (from) {
    const div = document.createElement('div')
    const fragment = DOMSerializer
      .fromSchema(this.schema)
      .serializeFragment(from)
    div.appendChild(fragment)
    return div.innerHTML
  }
}

document.addEventListener('DOMContentLoaded', function () {
  const godoTextareas = document.querySelectorAll('textarea[data-godo-schema]')
  godoTextareas.forEach((textarea) => {
    let options = {}
    options.schema = textarea.dataset.godoSchema
    options.instantUpdate = textarea.dataset.godoInstantUpdate !== undefined
    options.updateEvent = textarea.dataset.godoUpdateEvent
    if (textarea.dataset.godoHeadingLevels) {
      options.headingLevels = textarea.dataset.godoHeadingLevels
    }
    new Godo(textarea, options) // eslint-disable-line no-new
  })
})

